from ngsolve import *
from netgen.geom2d import unit_square

ngsglobals.msg_level = 1


mesh = Mesh(unit_square.GenerateMesh(maxh=0.2))
fes = H1(mesh, order=3, dirichlet=[1,2,3,4])

print ("ndof1 =", fes.ndof)
for el in mesh.Elements():
    fes.SetOrder(NodeId(CELL,el.nr), 4)
    for ed in el.edges:
        fes.SetOrder(ed, 4)
fes.UpdateDofTables()
print ("ndof2 =", fes.ndof)

u,v = fes.TnT()

f = LinearForm(fes)
f += SymbolicLFI(32 * (y*(1-y)+x*(1-x)) * v)

a = BilinearForm(fes, symmetric=True)
a += SymbolicBFI(grad(u)*grad(v))

a.Assemble()
f.Assemble()

gfu = GridFunction(fes)
gfu.vec.data = a.mat.Inverse(fes.FreeDofs(), inverse="sparsecholesky") * f.vec

Draw (gfu)
Draw (-grad(gfu), mesh, "Flux")

exact = 16*x*(1-x)*y*(1-y)
print ("L2-error:", sqrt (Integrate ( (gfu-exact)*(gfu-exact), mesh)))
